<?php

// DB_mysqli.class.php - abstract database functions and make them easier to use
// Attempts to broadly retain syntactic compatibility with PEAR DB class.
// MySQL version

// $Id: DB_mysqli.class.php 1882 2009-08-13 14:48:59Z tim $
//
// (c)2001-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require_once('mysqli.php');

/**
 * MySQLi driver.
 * See the source to the MySQL driver for code documentation.
 *
 */
class DB_mysqli {

	var $_dbh;					// Database handle
	var $_dbtype='mysqli';	// Database type
	var $_dbhost;				// Database host
	var $_dbname;				// Database name

	// Connect to the database server and select the database we wish to use
	function DB_mysqli ($dbhost, $username, $password, $dbname, $charset=null) {
		global $PHPOF_DB_CONNECTION_TYPE;
		
		if (!function_exists('mysqli_connect')) {
			trigger_error('MySQLi extension is not installed', E_USER_ERROR);
		}
		
		//pconnect doesn't seem to be supported in mysqli
		$this->_dbh = @mysqli_connect($dbhost, $username, $password, $dbname);
		
		if ($this->_dbh) {
			$this->_dbname = $dbname;
			$this->_dbhost = $dbhost;
			
			// Handle client character set
			if ($charset === null) $charset = 'latin1';
			if (version_compare(PHP_VERSION, '5.0.5', '>=')) {
				mysqli_set_charset($this->_dbh, $charset);
			} else {
				mysqli_query($this->_dbh, "SET NAMES $charset");
			}
			
		} else {
			trigger_error('Could not connect to database ('.mysql_error().')',E_USER_ERROR);
		}
	}

	// Query the database
	function query ($query) {
		global $PHPOF_DEBUG_MODE;
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysqli::query(): $query\n");
		$result = @mysqli_query ($this->_dbh, $query);
		
		// Sometimes $result can be false (due to, e.g. syntax error) but mysql_error() doesn't return anything
		// e.g. SELECT * FROM table WHERE <nonexistentfield>='%foo%'
		// Need to investigate this further
		/*if ($result == false) {
			trigger_error("mysql_query returned false (SQL query was: $query)",E_USER_ERROR);
		}*/
		
		if ($err = mysqli_error($this->_dbh)) {
			$err .= " (SQL query was: $query)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		

		//die('error');
		return $resobj;
	}
	
	// Perform a query, returning results from a particular page
	function query_paged ($query, $results_per_page, $page) {
		global $PHPOF_DEBUG_MODE;
		$base = $results_per_page * ($page - 1);
		$fullquery = "$query LIMIT $base, $results_per_page";
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query_paged(): $fullquery\n");
		
		$result = @mysqli_query ($this->_dbh, $fullquery);
		
		if ($err = mysqli_error($this->_dbh)) {
			$err .= " (SQL query was: $fullquery)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		$resobj->pages = $this->query_num_pages($query, $results_per_page);
		
		return $resobj;
	}
	
	// Get the number of pages that would be returned for a given query
	function query_num_pages ($query, $results_per_page) {
		global $PHPOF_DEBUG_MODE;
		if ($results_per_page == 0) trigger_error('DB_mysql::query_num_pages() called with zero results per page',E_USER_ERROR);
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_mysql::query_num_pages(): $query\n");
		$result = @mysqli_query ($this->_dbh, $query);
		
		if ($err = mysqli_error($this->_dbh)) {
			$err .= " (SQL query was: $query)";
			trigger_error($err,E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_dbh, $result);
		$rows = $resobj->numRows();
		
		return ceil ($rows / $results_per_page);
	}
	
	function beginTransaction()
	{
		$this->query('BEGIN');
	}
	
	function rollbackTransaction()
	{
		$this->query('ROLLBACK');

	}
	
	function commitTransaction()
	{
		$this->query('COMMIT');
	}
	
	function qstr($String)
	{
		return mysqli_real_escape_string($this->_dbh, $String);
	}
}

class DB_result {

	var $_dbh;
	var $_rr;
	var $pages=1;	// total pages for this query

	function DB_result (&$dbh, $rr) {
		$this->_dbh = &$dbh;
		$this->_rr = $rr;
	}

	// Get the number of rows in the result
	function numRows () {
		return mysqli_num_rows($this->_rr);
	}

	function fetchRow () {
		return mysqli_fetch_assoc($this->_rr);
	}

	function insertedId ($table_name='') {	// $table_name not used for MySQL
		return mysqli_insert_id($this->_dbh);
	}
	
	function getArray ($key, $value=PHPOF_ARRAY_LINEAR) {
		while ($result = $this->fetchRow()) {
			if ($value == PHPOF_ARRAY_LINEAR) {
				$array[] = $result[$key];
			} else {
				$array[$result[$key]] = $result[$value];
			}
		}
		if (isset($array)) {
			return $array;
		}
		return false;
	}
	
	// $Index is the name of the column to be used as an index, or PHPOF_USE_KEY
	// $Expression is an expression to be evaluated in the context of a DBRow object
	function getArrayWithDN($RowObject, $Index=PHPOF_USE_KEY, $Expression=PHPOF_USE_DEFAULT_DN, $OrderBy=PHPOF_USE_DEFAULT_ORDER) {
		
		$ReturnArray = null;
		
		if ($Expression != PHPOF_USE_DEFAULT_DN) {
			$RowObject->DescriptiveNameExpression = $Expression;
		}
		
		if (empty($RowObject->DescriptiveNameExpression)) trigger_error("DB_result::getArrayDN() called with request to use DBRow DescriptiveName (DN) as result value, but DBRow object does not have a DescriptiveNameExpression defined");
		
		if ($Index == PHPOF_USE_KEY) {
			$Index = $RowObject->_table->_key;
		}
		
		while ($this->fetchIntoObject($RowObject)) {
			$ReturnArray[$RowObject->$Index] = $RowObject->DescriptiveName;
		}
		return $ReturnArray;
	}

	function fetchIntoObject (&$object_to_fill) {
		global $PHPOF_AUDIT_LOGGING;
		if (!is_object($object_to_fill)) trigger_error('DB_result::fetchIntoObject() was not passed an object',E_USER_ERROR);
		if (!empty($object_to_fill->_Hook_FetchIntoObject_pre)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_pre);
		
		if ($row = $this->fetchRow()) {
			foreach ($row as $field => $value) {
				$object_to_fill->$field = $value;
			}
			if ($PHPOF_AUDIT_LOGGING) $object_to_fill->_storeAuditData();
			
			// Set row object descriptive name
			$object_to_fill->setDescriptiveName();
			
			$object_to_fill->resolveBinaryFields();
			
			// Call post-hook
			if (!empty($object_to_fill->_Hook_FetchIntoObject_post)) eval('$object_to_fill->'.$object_to_fill->_Hook_FetchIntoObject_post);
			return true;
		} else {
			return false;
		}
	}

	function affectedRows () {
		return mysqli_affected_rows ($this->_dbh);
	}
	
}
