<?php
// mysqli.php - MySQLi support module for PHPOF
//
// $Id: mysqli.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// Boolean types
define('DB_BOOLEAN_TRUE', '1');
define('DB_BOOLEAN_FALSE', '0');

function _DBTable_getFields (&$object)
{
	global $PHPOF_MYSQL_DISABLE_TRANSACTION_CHECKING;
	
	$object->_fields = array(); // Initialise rather than unset!
	$object->_fieldlist = '';	// Initialise rather than unset() to stop E_NOTICE errors when appending
	unset($object->_binaryfields);
	$object->_binaryfieldlist = '';
	
	if (!is_object($object)) trigger_error('MySQL _DBTable_GetFields() called with non-object',E_USER_ERROR);
	if (!is_object($object->_db)) trigger_error('MySQL _DBTable_GetFields() called with object without initialised DB',E_USER_ERROR);
	
	$res = $object->_db->query("DESCRIBE $object->_name");
	while ($field = $res->fetchRow()) {
		$fieldname = $field['Field'];
		if ($field['Key'] != 'PRI') {								// Make sure the field is NOT the primary key
			if (!stristr($field['Type'],'blob')) {				// If the field is not a binary field...
				$object->_fields[] = $fieldname;						// ...then add it to 
				$object->_fieldlist .= "$fieldname,";				// ...the field lists
			} else {																			// otherwise
				$object->_binaryfields[] = $fieldname;			// ... add it to the list of binary fields
				$object->_binaryfieldlist .= "$fieldname,";	// ...
			}
			$object->_fieldtype[$fieldname] = $field['Type'];
		} else {																				// We have found the table's primary key
			$object->_key = $field['Field'];
			if (stristr($field['Extra'],'auto_increment')) $object->_key_auto_increment = true; 
		}
		if ($field['Null'] == 'YES') {
			$object->allownull[$fieldname] = true;
		} else {
			$object->allownull[$fieldname] = false;
		}
	}
	// Strip trailing comma from field lists
	if (isset($object->_fieldlist)) $object->_fieldlist = substr($object->_fieldlist, 0, -1);
	if (isset($object->_binaryfieldlist)) $object->_binaryfieldlist = substr($object->_binaryfieldlist, 0, -1);
	
	// Find whether this table supports transactions
	if (!$PHPOF_MYSQL_DISABLE_TRANSACTION_CHECKING) {
		$res = $object->_db->query("SHOW TABLE STATUS LIKE '$object->_name'");
		$row = $res->fetchRow();
		if (isset($row['Type']) && ($row['Type'] == 'InnoDB')) $object->SupportsTransactions = true;
	}
}

function _DBTable_getColumnInfo (&$Table, &$Column)
{
	if (!empty($Table->_fieldtype[$Column->Name])) {
		
		// Find MySQL data type and constraints
		preg_match("/([a-z]+)(\((.+)\))?/i", $Table->_fieldtype[$Column->Name], $Matches);
		list ($Null,$ColTypeMySQL) = $Matches;
		
		// Constraints
		if (isset($Matches[3])) $Column->Constraints = $Matches[3];
		
		// MySQL type
		$ColTypeMySQL = strtolower($ColTypeMySQL);
		
		// Translate MySQL type to PHPOF type
		switch ($ColTypeMySQL) {
		case 'char':
		case 'varchar':
			$Column->Type = DBTABLE_COLTYPE_STRING;
			break;
		case 'tinyint':
			if ($Column->Constraints == '1') {
				$Column->Type = DBTABLE_COLTYPE_BOOL;	// NB. We are assuming that TINYINT(1) is a bool
			} else {
				$Column->Type = DBTABLE_COLTYPE_INT;
			}
			break;
		case 'smallint':
		case 'mediumint':
		case 'bigint':
			$Column->Type = DBTABLE_COLTYPE_INT;
			break;
		case 'tinytext':
		case 'text':
		case 'mediumtext':
		case 'longtext':
			$Column->Type = DBTABLE_COLTYPE_TEXT;
			break;
		case 'enum':
			$Column->Type = DBTABLE_COLTYPE_ENUM;
			
			// Strip off closing/ending quotes for each item
			$Params = explode(',',$Column->Constraints);
			if (count($Params) > 0) {
				foreach ($Params as $Key => $Value) {
					$Params[$Key] = substr($Value,1,-1);
				}
			}
			$Column->Constraints = implode(',',$Params);
			break;
		case 'set':
			$Column->Type = DBTABLE_COLTYPE_SET;
			break;
		case 'tinyblob':
		case 'blob':
		case 'mediumblob':
		case 'longblob':
			$Column->Type = DBTABLE_COLTYPE_BLOB;
			break;
		case 'float':
		case 'double':
			$Column->Type = DBTABLE_COLTYPE_REAL;
			break;
		default:
			$Column->Type = DBTABLE_COLTYPE_UNKNOWN;
		}
		
		return $Column;
	}
	return false;
}

// TODO: add support for foreign keys if InnoDB in use
function _DBTable_getForeignKeys_db (&$Table)
{
	return PHPOF_NOT_SUPPORTED;
}

function _DBRow_updateBinaryData (&$row, $dest_column_name_data, $dest_column_name_type, $filename, $filetype='', &$dbh)
{
	if ($fd = fopen($filename,'r')) {
		$filesize = filesize($filename);
		$SQL = 'UPDATE '.$row->_table->_name.
			" SET $dest_column_name_data='".mysqli_real_escape_string( $dbh, fread($fd,$filesize) )."'";
			if (!empty($dest_column_name_type)) {
				$SQL .= ", $dest_column_name_type='$filetype' ";
			}
			$SQL .= 'WHERE '.$row->_table->_key."='".mysqli_real_escape_string($dbh, $row->{$row->_table->_key})."'";
		fclose($fd);
		$row->_table->_db->query($SQL);
	} else {
		trigger_error("Error in _DBRow_UpdateBinaryData(): could not open temp file $filename.",E_USER_ERROR);
	}
}

function _DBRow_updateBinaryDataFromBuffer (&$row, $dest_column_name_data, $dest_column_name_type, &$Data, $filetype='', &$dbh)
{
	$SQL = 'UPDATE '.$row->_table->_name.
		" SET $dest_column_name_data='".mysqli_real_escape_string($dbh, $Data)."' ";
		if (!empty($dest_column_name_type)) {
			$SQL .= ", $dest_column_name_type='$filetype' ";
		}
		$SQL .= 'WHERE '.$row->_table->_key."='".mysqli_real_escape_string($dbh, $row->{$row->_table->_key})."'";
	
	$row->_table->_db->query($SQL);
}

function _DBRow_RetrieveBinaryData (&$row, $column_name, $resultarray)
{
	return $resultarray[$column_name];
}

function _DBRow_RemoveBinaryData (&$row, $column_name, &$dbh)
{
	$row->_table->_db->query(
		'UPDATE '.$row->_table->_name.
		" SET $column_name='' WHERE ".$row->_table->_key."='".mysqli_real_escape_string($dbh, $row->{$row->_table->_key})."'");
}

